require 'spec_helper'

describe HmacSignature do
	let(:timestamp)			{ Time.at(2345) }
	let(:method)			{ 'GET' }
	let(:path)				{ '/test/path' }
	let(:params)			{{:param1 => 'param1', :param2 => 'param2'}}
	let(:key)				{ 'key123' }
	let(:secret)			{ 'secret123' }
	let(:version)			{ HmacSignature::Signature::VERSION }
	let(:expiry)			{ (timestamp.to_i + 600).to_s }
	let(:signature)			{ '00d639f0c7561b976893deed5360b339a90971aa8bd8e1bd9f1aab30f303ac3a' }

	let(:token_key)			{ key }
	let(:token_secret)		{ secret }
	let(:token) 			{ HmacSignature::Token.new token_key, token_secret }

	let(:cred_key)			{ key }
	let(:cred_expiry)		{ expiry }
	let(:cred_version)		{ version }
	let(:cred_signature)	{ signature }
	let(:credentials)		{ HmacSignature::Credentials.new cred_key, cred_expiry, cred_version, cred_signature }

	let(:req_headers_a)		{{ 'X-Auth-Key' => key, 'X-Auth-Expiry' => expiry, 'X-Auth-Version' => version, 'X-Auth-Signature' => signature, 'Content-Type' => 'text/plain', 'Content-Length' => 12345 }}
	let(:req_headers_b)		{{ 'HTTP_X_AUTH_KEY' => key, 'HTTP_X_AUTH_EXPIRY' => expiry, 'HTTP_X_AUTH_VERSION' => version, 'HTTP_X_AUTH_SIGNATURE' => signature, 'HTTP_CONTENT_TYPE' => 'text/plain', 'HTTP_CONTENT_LENGTH' => 12345 }}
	let(:req_method)		{ method }
	let(:req_path)			{ path }
	let(:req_params)		{ params }
	let(:req_credentials)	{ credentials }
	let(:request)			{ HmacSignature::Request.new req_method, req_path, req_params, req_credentials}

	before do
		Time.stub!(:now).and_return(timestamp)
	end

	describe HmacSignature::Token do
	end

	describe HmacSignature::Credentials do
		describe "::from_params" do
			context "when symbol keys" do
				hash = {:blah => 'blah', :auth_key => 'key', :auth_expiry => 'expiry', :auth_version => 'version', :auth_signature => 'signature' }
				credentials = HmacSignature::Credentials.from_params(hash)
				credentials.key.should == 'key'
				credentials.expiry.should == 'expiry'
				credentials.version.should == 'version'
				credentials.signature.should == 'signature'
			end

			context "when string keys" do
			end
		end

		describe "::from_headers" do
			subject(:response){ HmacSignature::Credentials.from_headers(headers) }
			
			context "when headers in normal form" do
				let(:headers){ req_headers_a }

				its(:key){ should == key }
				its(:expiry){ should == expiry }
				its(:expiry){ should == expiry }
				its(:version){ should == version }
				its(:signature){ should == signature }
			end

			context "when headers in normal form" do
				let(:headers){ req_headers_b }

				its(:key){ should == key }
				its(:expiry){ should == expiry }
				its(:expiry){ should == expiry }
				its(:version){ should == version }
				its(:signature){ should == signature }
			end
		end
	
		describe "#to_hash" do
			it "returns a credential hash" do
				credentials.to_hash.should == {
					"auth_key" => key, 
					"auth_expiry" => "2945", 
					"auth_version" => version, 
					"auth_signature" => signature
				}
			end
		end
	end

	describe HmacSignature::Signature do
		subject{ HmacSignature::Signature.new method, path, params, expiry, version, key, secret }
		its(:to_s){ should == signature }
	end

	describe HmacSignature::Request do
		describe "::new" do
			context "when method is nil" do
				let(:req_method){ nil }
				it "raises error" do
					expect{ request }.to raise_error(ArgumentError, 'Expected string')
				end
			end

			context "when method is empty string" do
				let(:req_method){ '' }
				it "raises error" do
					expect{ request }.to raise_error(ArgumentError, 'Expected string')
				end
			end

			context "when path is nil" do
				let(:req_path){ nil }
				it "raises error" do
					expect{ request }.to raise_error(ArgumentError, 'Expected string')
				end
			end

			context "when path is empty string" do
				let(:req_path){ '' }
				it "raises error" do
					expect{ request }.to raise_error(ArgumentError, 'Expected string')
				end
			end

			context "when params is not a hash" do
				let(:req_params){ '' }
				it "raises error" do
					expect{ request }.to raise_error(ArgumentError, 'Expected hash')
				end
			end

			context "when credentials is not an instance of Credentials" do
				let(:req_credentials){ '' }
				it "raises error" do
					expect{ request }.to raise_error(ArgumentError, 'Expected credentials')
				end
			end
		end

		describe "#sign" do
			it "returns credentials instance" do
				creds = request.sign(token)
				creds.expiry.should == expiry
				creds.signature.should == signature
				creds.version.should == version
				creds.key.should == key
			end
		end

		describe "authenticate_by_token!" do
			subject(:auth){ request.authenticate_by_token! token }

			context "when params are valid" do
				it "returns true" do
					auth.should be_true
				end
			end

			context "when token is missing secret" do
				it "raises exception" do
					tok = HmacSignature::Token.new token_key, ''
					expect do
						request.authenticate_by_token! tok
					end.to raise_error(ArgumentError, 'Provided token is missing secret')
				end
			end

			context "when signature is invalid" do
				let(:cred_signature) { 'incorrect123' }
				it "returns true" do
					expect { auth }.to raise_error(HmacSignature::AuthenticationError, 'Invalid signature: you should have sent HmacSHA256Hex("GET\n/test/path\nauth_expiry=2945&auth_key=key123&auth_version=1.0&param1=param1&param2=param2", your_secret_key), but you sent "incorrect123"')
				end
			end

			context "when expiry timestamp is expired" do
				let(:req_credentials) do
					HmacSignature::Request.new(req_method, req_path, req_params).sign(token, -600)
				end

				it "returns true" do
					expect { auth }.to raise_error(HmacSignature::AuthenticationError, 'Timestamp expired: Given timestamp (1970-01-01T00:29:05Z) has expired. The current server time is (1970-01-01T00:39:05Z)')
				end
			end

			context "when expiry timestamp is really expired" do
				let(:req_credentials) do
					HmacSignature::Request.new(req_method, req_path, req_params).sign(token, -4600)
				end

				it "returns true" do
					expect { auth }.to raise_error(HmacSignature::AuthenticationError, 'Timestamp expired: Given timestamp (1969-12-31T23:22:25Z) has expired. The current server time is (1970-01-01T00:39:05Z)')
				end
			end

			context "when expiry timestamp is slightly expired" do
				let(:req_credentials) do
					HmacSignature::Request.new(req_method, req_path, req_params).sign(token, -10)
				end

				it "returns true" do
					expect { auth }.to raise_error(HmacSignature::AuthenticationError, 'Timestamp expired: Given timestamp (1970-01-01T00:38:55Z) has expired. The current server time is (1970-01-01T00:39:05Z)')
				end
			end
		end
	end

	describe HmacSignature::Strategy::Headers::Request do
		let(:header_request) { HmacSignature::Strategy::Headers::Request.new req_method, req_path, req_params, req_headers_a }
		describe "#sign" do
			it "returns a hash of header values" do
				header_request.sign(token).should == {
					"X-Auth-Key"=> key, 
					"X-Auth-Expiry"=>"2945", 
					"X-Auth-Version"=> version, 
					"X-Auth-Signature"=> signature
				}
			end
		end

		describe "#authenticate_by_token!" do
			it "returns true" do
				header_request.authenticate_by_token!(token).should be_true
			end
		end
	end

	describe HmacSignature::Strategy::Params::Request do
		let(:params_request_params) { params.merge credentials.to_hash }
		let(:params_request) { HmacSignature::Strategy::Params::Request.new req_method, req_path, params_request_params }
		describe "#sign" do
			it "returns a hash of header values" do
				params_request.sign(token).should == {
					"auth_key"=> key, 
					"auth_expiry"=>"2945", 
					"auth_version"=> version, 
					"auth_signature"=> signature
				}
			end
		end

		describe "#authenticate_by_token!" do
			it "returns true" do
				params_request.authenticate_by_token!(token).should be_true
			end
		end
	end
end